Imports SpeechLib

Public Class ScreenReader
    Public voice As New SpVoice
    Public currentnode As Node
    Public history As New ArrayList
    Public mode As Boolean = False

    Public Sub New()
        voice.Priority = SpeechVoicePriority.SVPAlert
    End Sub

    Public Sub New(ByVal n As Node)
        'voice.Priority = SpeechVoicePriority.SVPOver
        'voice.WaitUntilDone(100)
        setCurrentNode(n)
        speakSelected()
    End Sub

    Public Sub speak(ByVal s As String)
        voice.Speak(s, SpeechLib.SpeechVoiceSpeakFlags.SVSFlagsAsync Or SpeechLib.SpeechVoiceSpeakFlags.SVSFPurgeBeforeSpeak)
    End Sub

    Public Sub speak()
        speak(currentnode.getValue())
    End Sub

    Public Sub speakSelected()
        If Not currentnode.getSelectedNode Is Nothing Then
            speak(currentnode.getSelectedNode.getValue)
        End If
    End Sub

    Public Sub descend()
        If Not currentnode.getSelectedNode() Is Nothing Then
            If (currentnode.getSelectedNode.nodes.Count > 0 Or mode) Then
                history.Add(currentnode)
                setCurrentNode(currentnode.getSelectedNode())
            ElseIf currentnode.getSelectedNode().onDescendInto() = 0 Then
                Beep()
            End If
        Else
            Beep()
        End If

        currentnode.onDescendInto()
    End Sub

    Public Sub ascend()
        If history.Count > 0 Then
            Dim n As Node = history(history.Count - 1)
            history.RemoveAt(history.Count - 1)
            setCurrentNode(n)
        Else
            Beep()
        End If
    End Sub

    Public Sub showHistory()
        Dim s As String = ""

        For i As Integer = 0 To history.Count - 1
            s += history(i).value & " > "
        Next

        MessageBox.Show(s)
    End Sub

    Public Sub setCurrentNode(ByVal n As Node)
        If n Is Nothing Or n.deleted Then
            ascend()
        Else
            currentnode = n
        End If
    End Sub

    Public Sub update()
        setCurrentNode(currentnode)
        speakSelected()
    End Sub

    Public Sub interpret(ByVal c As Char)
        If mode Then
            If c = Chr(27) Then
                mode = False
                ascend()
                speakSelected()
            Else
                speak(c)
                currentnode.KeyPress(c)
            End If
        Else
            If c = "8" Then
                currentnode.selectedrow -= 1
            ElseIf c = "2" Then
                currentnode.selectedrow += 1
            ElseIf c = "4" Then
                currentnode.selectedcol -= 1
            ElseIf c = "6" Then
                currentnode.selectedcol += 1
            ElseIf c = "5" Then
                descend()
            ElseIf c = "0" Then
                ascend()
            ElseIf c = Chr(13) And currentnode.getSelectedNode().editable Then
                mode = True
                descend()
                currentnode.value = ""
                speakSelected()
            Else
                speak(c)
            End If

            If currentnode.selectedcol < currentnode.getLowestCol() Then
                currentnode.selectedcol = currentnode.getLowestCol()
                Beep()
            End If

            If currentnode.selectedcol > currentnode.getHighestCol() Then
                currentnode.selectedcol = currentnode.getHighestCol()
                Beep()
            End If


            If currentnode.selectedrow < currentnode.getLowestRow() Then
                currentnode.selectedrow = currentnode.getLowestRow()
                Beep()
            End If

            If currentnode.selectedrow > currentnode.getHighestRow() Then
                currentnode.selectedrow = currentnode.getHighestRow()
                Beep()
            End If

            speakSelected()
        End If
    End Sub
End Class
